C $Header: /u/u0/gcmpack/MITgcm/model/src/solve_for_pressure.F,v 1.30 2002/03/04 17:26:41 adcroft Exp $
C $Name:  $

#include "CPP_OPTIONS.h"

CBOP
C     !ROUTINE: SOLVE_FOR_PRESSURE
C     !INTERFACE:
      SUBROUTINE SOLVE_FOR_PRESSURE(myTime, myIter, myThid)

C     !DESCRIPTION: \bv
C     *==========================================================*
C     | SUBROUTINE SOLVE_FOR_PRESSURE                             
C     | o Controls inversion of two and/or three-dimensional      
C     |   elliptic problems for the pressure field.               
C     *==========================================================*
C     \ev

C     !USES:
      IMPLICIT NONE
C     == Global variables
#include "SIZE.h"
#include "EEPARAMS.h"
#include "PARAMS.h"
#include "DYNVARS.h"
#include "GRID.h"
#include "SURFACE.h"
#include "FFIELDS.h"
#ifdef ALLOW_NONHYDROSTATIC
#include "SOLVE_FOR_PRESSURE3D.h"
#include "GW.h"
#endif
#ifdef ALLOW_OBCS
#include "OBCS.h"
#endif
#include "SOLVE_FOR_PRESSURE.h"

C     !INPUT/OUTPUT PARAMETERS:
C     == Routine arguments ==
C     myTime - Current time in simulation
C     myIter - Current iteration number in simulation
C     myThid - Thread number for this instance of SOLVE_FOR_PRESSURE
      _RL myTime
      INTEGER myIter
      INTEGER myThid

C     !LOCAL VARIABLES:
C     == Local variables ==
      INTEGER i,j,k,bi,bj
      _RS uf(1-Olx:sNx+Olx,1-Oly:sNy+Oly)
      _RS vf(1-Olx:sNx+Olx,1-Oly:sNy+Oly)
      _RL firstResidual,lastResidual
      _RL tmpFac
      INTEGER numIters
      CHARACTER*(MAX_LEN_MBUF) msgBuf
C     Temps for calculating pressure gradient that ensure div(U) == 0.
      _RL phiSurfX(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL phiSurfY(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      INTEGER iMin,iMax
      INTEGER jMin,jMax
      CHARACTER*(MAX_LEN_FNAM) fn
      INTEGER prec
      LOGICAL lgf

CEOP
C     Read meanu into gUNm1 and meanv into gVNM1 here.
      prec = precFloat32
      WRITE(fn,'(A,I10.10)') 'uVelMeanIn.',10101
      CALL MDSREADFIELD(fn,prec,'RL',Nr,gUNm1,   1,myThid)
      WRITE(fn,'(A,I10.10)') 'vVelMeanIn.',10101
      CALL MDSREADFIELD(fn,prec,'RL',Nr,gVNm1,   1,myThid)

C     Do exchange to ensure that gUNM1 and gVNM1 are correct in overlaps.
      _EXCH_XYZ_RL(gUNm1 , myThid )
      _EXCH_XYZ_RL(gVNm1 , myThid )
C      DO bj=myByLo(myThid),myByHi(myThid)
C       DO bi=myBxLo(myThid),myBxHi(myThid)
C        DO K=Nr,1,-1
C         DO j=1,sNy+1
C          DO i=1,sNx+1
C           IF ( ABS(gUNM1(i,j,k,bi,bj)) .GT. 1.5 ) 
C     &      gUNM1(i,j,k,bi,bj) = 0.
C           IF ( ABS(gVNM1(i,j,k,bi,bj)) .GT. 1.5 ) 
C     &      gVNM1(i,j,k,bi,bj) = 0.
C          ENDDO
C         ENDDO
C        ENDDO
C       ENDDO
C      ENDDO

      CALL PLOT_FIELD_XYZRL ( gUNm1 , ' gUNm1' ,
     &       Nr, myIter, myThid )
      CALL PLOT_FIELD_XYZRL ( gVNm1 , ' gVNm1' ,
     &       Nr, myIter, myThid )


C     Set etaN to zero.
      DO bj=myByLo(myThid),myByHi(myThid)
       DO bi=myBxLo(myThid),myBxHi(myThid)
        DO j=1-OLy,sNy+OLy
         DO i=1-OLx,sNx+OLx
          etaN(i,j,bi,bj) = 0.
         ENDDO
        ENDDO
       ENDDO
      ENDDO

C--   Save previous solution & Initialise Vector solution and source term :
      DO bj=myByLo(myThid),myByHi(myThid)
       DO bi=myBxLo(myThid),myBxHi(myThid)
        DO j=1-OLy,sNy+OLy
         DO i=1-OLx,sNx+OLx
#ifdef INCLUDE_CD_CODE
          etaNm1(i,j,bi,bj) = etaN(i,j,bi,bj)
#endif
          cg2d_x(i,j,bi,bj) = Bo_surf(i,j,bi,bj)*etaN(i,j,bi,bj)
          cg2d_b(i,j,bi,bj) = 0.
         ENDDO
        ENDDO
        IF (useRealFreshWaterFlux) THEN
         tmpFac = freeSurfFac
         IF (exactConserv) tmpFac = freeSurfFac*implicDiv2DFlow
         DO j=1,sNy
          DO i=1,sNx
           cg2d_b(i,j,bi,bj) = 
     &       tmpFac*_rA(i,j,bi,bj)*EmPmR(i,j,bi,bj)/deltaTMom
          ENDDO
         ENDDO
        ENDIF
       ENDDO
      ENDDO

C     Setup RHS
      DO bj=myByLo(myThid),myByHi(myThid)
       DO bi=myBxLo(myThid),myBxHi(myThid)
        DO K=Nr,1,-1
         DO j=1,sNy+1
          DO i=1,sNx+1
           uf(i,j) = _dyG(i,j,bi,bj)
     &      *drF(k)*_hFacW(i,j,k,bi,bj)
           vf(i,j) = _dxG(i,j,bi,bj)
     &      *drF(k)*_hFacS(i,j,k,bi,bj)
          ENDDO
         ENDDO
         CALL CALC_DIV_GHAT(
     I       bi,bj,1,sNx,1,sNy,K,
     I       uf,vf,
     U       cg2d_b,
     I       myThid)
        ENDDO
       ENDDO
      ENDDO

#ifndef DISABLE_DEBUGMODE
      IF (debugMode) THEN
       CALL DEBUG_STATS_RL(1,cg2d_b,'cg2d_b (SOLVE_FOR_PRESSURE)',
     &                        myThid)
      ENDIF
#endif
      prec = precFloat32
      lgf  = globalFiles
      WRITE(fn,'(A,I10.10)') 'cg2d_b.',10101
      CALL MDSWRITEFIELD(fn,prec,lgf,'RL',1,
     &      cg2d_b,1,myIter,myThid)

C--   Find the surface pressure using a two-dimensional conjugate
C--   gradient solver.
C     see CG2D.h for the interface to this routine.
      firstResidual=0.
      lastResidual=0.
      numIters=cg2dMaxIters
      CALL CG2D(
     U           cg2d_b,
     U           cg2d_x,
     O           firstResidual,
     O           lastResidual,
     U           numIters,
     I           myThid )
      _EXCH_XY_R8(cg2d_x, myThid )

#ifndef DISABLE_DEBUGMODE
      IF (debugMode) THEN
       CALL DEBUG_STATS_RL(1,cg2d_x,'cg2d_x (SOLVE_FOR_PRESSURE)',
     &                        myThid)
      ENDIF
#endif

      _BEGIN_MASTER( myThid )
      WRITE(msgBuf,'(A34,1PE24.14)') 'cg2d_init_res =',firstResidual
      CALL PRINT_MESSAGE(msgBuf,standardMessageUnit,SQUEEZE_RIGHT,1)
      WRITE(msgBuf,'(A34,I6)') 'cg2d_iters =',numIters
      CALL PRINT_MESSAGE(msgBuf,standardMessageUnit,SQUEEZE_RIGHT,1)
      WRITE(msgBuf,'(A34,1PE24.14)') 'cg2d_res =',lastResidual
      CALL PRINT_MESSAGE(msgBuf,standardMessageUnit,SQUEEZE_RIGHT,1)
      _END_MASTER( )

C--   Transfer the 2D-solution to "etaN" :
      DO bj=myByLo(myThid),myByHi(myThid)
       DO bi=myBxLo(myThid),myBxHi(myThid)
        DO j=1-OLy,sNy+OLy
         DO i=1-OLx,sNx+OLx
          etaN(i,j,bi,bj) = recip_Bo(i,j,bi,bj)*cg2d_x(i,j,bi,bj)
         ENDDO
        ENDDO
       ENDDO
      ENDDO

      DO bj=myByLo(myThid),myByHi(myThid)
       DO bi=myBxLo(myThid),myBxHi(myThid)

C--     Set up work arrays that need valid initial values
        DO j=1-OLy,sNy+OLy
         DO i=1-OLx,sNx+OLx
          phiSurfX(i,j)=0.
          phiSurfY(i,j)=0.
         ENDDO
        ENDDO

C       Loop range: Gradients of Eta are evaluated so valid
C       range is all but first row and column in overlaps.
        iMin = 1-OLx+1
        iMax = sNx+OLx
        jMin = 1-OLy+1
        jMax = sNy+OLy

C-      Calculate gradient of surface Potentiel
        CALL CALC_GRAD_PHI_SURF(
     I       bi,bj,iMin,iMax,jMin,jMax,
     I       etaN,
     O       phiSurfX,phiSurfY,
     I       myThid )


C       Step forward zonal velocity
        DO K=1,Nr
        DO j=jMin,jMax
         DO i=iMin,iMax
          uVel(i,j,k,bi,bj)=( gUNm1(i,j,k,bi,bj)
     &         -deltaTmom*implicSurfPress*phiSurfX(i,j)
     &                      )*_maskW(i,j,k,bi,bj)
         ENDDO
        ENDDO

C       Step forward meridional velocity
        DO j=jMin,jMax
         DO i=iMin,iMax
          vVel(i,j,k,bi,bj)=( gVNm1(i,j,k,bi,bj)
     &       -deltaTmom*implicSurfPress*phiSurfY(i,j)
     &                    )*_maskS(i,j,k,bi,bj)
         ENDDO
        ENDDO
        ENDDO

       ENDDO
      ENDDO

C     exchange to update edges
      _EXCH_XYZ_RL(uVel , myThid )
      _EXCH_XYZ_RL(vVel , myThid )

      CALL PLOT_FIELD_XYZRL ( vVel , ' vVel' ,
     &       Nr, myIter, myThid )
      CALL PLOT_FIELD_XYZRL ( gVnm1 , ' gVnm1' ,
     &       Nr, myIter, myThid )

C     write out updated meanu and meanv
C       _BEGIN_MASTER( myThid )
C       _BARRIER

      prec = precFloat32
      lgf  = globalFiles
      WRITE(fn,'(A,I10.10)') 'etaCorrOut.',10101
      CALL MDSWRITEFIELD(fn,prec,lgf,'RL',1,
     &      etaN,1,myIter,myThid)
      WRITE(fn,'(A,I10.10)') 'uMeanOut.',10101
      CALL MDSWRITEFIELD(fn,prec,lgf,'RL',Nr,
     &      uVel,1,myIter,myThid)
      WRITE(fn,'(A,I10.10)') 'vMeanOut.',10101
      CALL MDSWRITEFIELD(fn,prec,lgf,'RL',Nr,
     &      vVel,1,myIter,myThid)

C       _END_MASTER( myThid )
       _BARRIER

C      STOP 'Ending within solve_for_pressure.F'

      RETURN
      END
