#include "CPP_OPTIONS.h"

C     !ROUTINE: OFFLINE_STEP
      SUBROUTINE OFFLINE_STEP( iloop, myTime, myIter, myThid )

C     *==================================================================
C     | SUBROUTINE offline_step
C     | o Run the offline code                                          
C     *==================================================================

      IMPLICIT NONE
C     == Global variables ==
#include "SIZE.h"
#include "EEPARAMS.h"
#include "PARAMS.h"
#include "DYNVARS.h"
#include "FFIELDS.h"
#ifdef OFFLINE_MODEL
#include "OFFLINE.h"
#endif
#ifdef ALLOW_PTRACERS
#include "PTRACERS.h"
#include "PTRACERS_OPTIONS.h"
#include "PTRACERS_STATV.h"
#endif

      LOGICAL  DIFFERENT_MULTIPLE
      EXTERNAL DIFFERENT_MULTIPLE

C     !LOCAL VARIABLES:
C     == Routine arguments ==
C     note: under the multi-threaded model myiter and 
C           mytime are local variables passed around as routine 
C           arguments. Although this is fiddly it saves the need to 
C           impose additional synchronisation points when they are 
C           updated.
C     myiter - iteration counter for this thread
C     mytime - time counter for this thread
C     mythid - thread number for this instance of the routine.
      integer iloop
      integer mythid 
      integer myIter
      _RL     mytime
c     == Local Variables ==
      CHARACTER*10 suff
      LOGICAL permCheckPoint

#ifdef OFFLINE_MODEL

c read in u,v,w,gmredi,etc fields
        call OFFLINE_FIELDS_LOAD( myTime, myIter, myThid )

#ifdef ALLOW_PTRACERS
       IF (usePTRACERS) THEN
c
         call OFFLINE_DYNAMICS(myTime, myIter, myThid)
c
         myIter = nIter0 + iLoop
         myTime = startTime + deltaTClock * float(iLoop)

         call OFFLINE_CORRECTION_STEP(myTime, myIter, myThid)
c 
         call PTRACERS_GTERM_BLOCKING_EXCHANGES( myThid )
c
         CALL PTRACERS_FIELDS_BLOCKING_EXCHANGES(myThid)
c
        IF (
     &     DIFFERENT_MULTIPLE(dumpFreq,myTime,
     &                     myTime-deltaTClock).OR.
     &     myTime.EQ.endTime .OR.
     &     myTime.EQ.startTime
     &   ) THEN
         _BARRIER
         _BEGIN_MASTER( myThid )
C--     Write ptracer fields 
         CALL PTRACERS_WRITE_STATE( myIter,myTime,myThid )
         _END_MASTER( myThid )
         _BARRIER
       ENDIF

#ifdef PTRACERS_SEPERATE_FORCING
cswdptr -- add for seperate timestepping of chemical/biological/forcing
cswdptr    of ptracers ---
c       call CHEM_FORCING(bi,bj,iMin,iMax,jMin,jMax,
c                         pTracer,s gPtr,
        call CHEM_FORCING(myTime,myIter,myThid )
cswdtr -- end add ---
#endif
c
cccccccccccccccccccccccc OUTPUT ccccccccccccccccccccccccccccccccc
c
#ifdef  ALLOW_TIMEAVE
         CALL PTRACERS_STATV_WRITE( myTime, myIter, myThid )
#endif
c
         permCheckPoint = .FALSE.
         permCheckPoint= DIFFERENT_MULTIPLE(pChkptFreq,myTime,
     &                    myTime-deltaTClock)

         IF ( (( permCheckPoint.OR.
     &     DIFFERENT_MULTIPLE(chkptFreq, myTime,myTime-deltaTClock)
     &     ) .AND. myIter.NE.nIter0 )
     &     ) THEN

          WRITE(suff,'(I10.10)') ,myIter
          call PTRACERS_WRITE_CHECKPOINT( suff,myIter,myTime,myThid )

         ENDIF
c
        ENDIF

#endif  /*ALLOW_PTRACER*/

#endif /* OFFLINE_MODEL */

      return
      END
