#!/bin/bash
# #PBS -l select=25:ncpus=8:model=har
#PBS -l select=25:ncpus=8
#PBS -N div_correct
#PBS -l walltime=2:00:00
#PBS -q devel@pbspl3
#PBS -m abe

VEL_DIR=/nobackupp1/rpaberna/global_tracer/tp_0.1_degree-anom
OUT_DIR=/nobackupp1/rpaberna/global_tracer/tp_0.1_degree-anom_corrected
RUN_DIR=/u/rpaberna/offline_model/global-AVISO/run

module purge
module load comp-intel/11.1.046 mpi/mpt.1.25 netcdf/3.6.0/intel

cd $RUN_DIR

for i in `seq 1 39`
do
    ITER=`printf '%010d' $i`
    
    # test if output files exist
    if [[ -e $OUT_DIR/uvel_tp_anom-corr.$ITER.data && -e $OUT_DIR/vvel_tp_anom-corr.$ITER.data ]]
    then
        echo "Files for $ITER already exist"
    else
        echo "Processing ITER $ITER"
        
        rm STD*
        rm *.data
        rm *.meta
        ln -s $VEL_DIR/uvel_tp_anom.$ITER.data uVelMeanIn.0000010101.data
        ln -s $VEL_DIR/vvel_tp_anom.$ITER.data vVelMeanIn.0000010101.data

        mpiexec ./mitgcmuv > output.log 2>&1

        mv -v uMeanOut.0000010101.data $OUT_DIR/uvel_tp_anom-corr.$ITER.data && 
        mv -v uMeanOut.0000010101.meta $OUT_DIR/uvel_tp_anom-corr.$ITER.meta &&
        mv -v vMeanOut.0000010101.data $OUT_DIR/vvel_tp_anom-corr.$ITER.data &&
        mv -v vMeanOut.0000010101.meta $OUT_DIR/vvel_tp_anom-corr.$ITER.meta
        # make sure it worked
        OUT=$?
        if [ $OUT -eq 0 ]; then
            echo "Copying files successfuly"
        else
            echo "Failed to find uMeanOut files"
            exit $OUT
        fi
    fi

done
