#include "CPP_OPTIONS.h"

C     !ROUTINE: OFFLINE_CORRECTION_STEP
      SUBROUTINE OFFLINE_CORRECTION_STEP(myTime, myIter, myThid)
C     *==========================================================*
C     | SUBROUTINE THE_CORRECTION_STEP                            
C     *==========================================================*
C     |1rst Part : Update Tracer
C     |
C     | The arrays used for time stepping are cycled.
C     | Tracers:
C     |           T(n) = Gt(n-1)
C     |           Gt(n-1) = Gt(n)
C     |
C     |part2: Adjustments.                                        
C     |   o Filter  (Shapiro Filter, Zonal_Filter)        
C     |   o Convective Adjustment                                 
C     |   o Diagmnostic of state variables (Time average)         
C     *==========================================================*

C     !USES:
      IMPLICIT NONE
C     == Global variables ===
#include "SIZE.h"
#include "EEPARAMS.h"
#include "PARAMS.h"
#include "DYNVARS.h"
#ifdef ALLOW_PTRACERS
#include "PTRACERS.h"
#endif

#ifdef ALLOW_SHAP_FILT
#include "SHAP_FILT.h"
#endif
#ifdef ALLOW_ZONAL_FILT   
#include "ZONAL_FILT.h"
#endif

C     !INPUT/OUTPUT PARAMETERS:
C     == Routine arguments ==
C     myTime - Current time in simulation
C     myIter - Current iteration number in simulation
C     myThid - Thread number for this instance of the routine.
      _RL myTime
      INTEGER myIter
      INTEGER myThid

#ifdef OFFLINE_MODEL
#ifdef ALLOW_PTRACERS

C     !LOCAL VARIABLES:
C     == Local variables
      INTEGER iMin,iMax
      INTEGER jMin,jMax
      INTEGER bi,bj
      INTEGER k,i,j


      DO bj=myByLo(myThid),myByHi(myThid)
       DO bi=myBxLo(myThid),myBxHi(myThid)


C       Loop range: Gradients of Eta are evaluated so valid
C       range is all but first row and column in overlaps.
        iMin = 1-OLx+1
        iMax = sNx+OLx
        jMin = 1-OLy+1
        jMax = sNy+OLy

C--     Loop over all layers, top to bottom
        DO K=1,Nr

C-        Update passive tracer fields:  T(n) = T**, Gt(n-1) = Gt(n)
              CALL PTRACERS_CYCLE(bi,bj,k,myIter,myTime,myThid)

        ENDDO

C--    End of 1rst bi,bj loop
       ENDDO
      ENDDO

C--- 2nd Part : Adjustment.
C
C       Static stability is calculated and the tracers are
C       convective adjusted where statically unstable.

C--   Filter (and exchange)
#ifdef ALLOW_SHAP_FILT
      IF (useSHAP_FILT) THEN
c       IF ( .NOT.(staggerTimeStep .AND. shap_filt_TrStagg) )
c     &  CALL SHAP_FILT_APPLY_TS( theta,salt, myTime, myIter, myThid )
      ENDIF
#endif 
#ifdef ALLOW_ZONAL_FILT
      IF (useZONAL_FILT) THEN
c      IF ( .NOT.(staggerTimeStep .AND. zonal_filt_TrStagg) )
c    &  CALL ZONAL_FILT_APPLY_TS( theta, salt, myThid )  

      ENDIF
#endif 

      DO bj=myByLo(myThid),myByHi(myThid)
       DO bi=myBxLo(myThid),myBxHi(myThid)

C--     Convectively adjust new fields to be statically stable
        iMin = 1-OLx+1
        iMax = sNx+OLx
        jMin = 1-OLy+1
        jMax = sNy+OLy
c       CALL CONVECTIVE_ADJUSTMENT(
c    I       bi, bj, iMin, iMax, jMin, jMax,
c    I       myTime, myIter, myThid )


#ifdef ALLOW_TIMEAVE
        IF (taveFreq.GT.0.) THEN 
          CALL PTRACERS_STATVARS(myTime, myIter, bi, bj, myThid)
        ENDIF
#endif /* ALLOW_TIMEAVE */

C--    End of 2nd bi,bj loop
       ENDDO
      ENDDO

#endif
#endif

      RETURN
      END
